@@ -3,10 +3,12 @@ import socket
 import tornado
 import tornado.ioloop
 import tornado.process
-import tornado.iostream as iostream
-import tornado.tcpserver as tcpserver
 
 from .routing import RoutingHandler
+
+from pyrox.util.streaming import IOStream, StreamClosedError
+from pyrox.util.sserver import TCPServer
+
 from pyrox.log import get_logger
 from pyrox.http import (HttpRequest, HttpResponse, RequestParser,
                         ResponseParser, ParserDelegate)
@@ -108,10 +110,12 @@ class DownstreamHandler(ProxyHandler):
         # Execute against the pipeline
         action = self._filter_pl.on_request_head(self._http_msg)
 
-        self._http_msg.remove_header('content-length')
-        self._http_msg.remove_header('transfer-encoding')
+        # If there's a content length, negotiate the tansfer encoding
+        if self._http_msg.get_header('content-length'):
+            self._http_msg.remove_header('content-length')
+            self._http_msg.remove_header('transfer-encoding')
 
-        self._http_msg.header('transfer-encoding').values.append('chunked')
+            self._http_msg.header('transfer-encoding').values.append('chunked')
 
         # If we're rejecting then we're not going to connect to upstream
         if action.is_rejecting():
@@ -130,22 +134,21 @@ class DownstreamHandler(ProxyHandler):
             accumulator = AccumulationStream()
             data = bytes[:length]
 
-            print(data)
-
             self._filter_pl.on_request_body(data, accumulator)
 
             if accumulator.size() > 0:
-                print('here')
                 data = accumulator.bytes
 
-            # If we're not already connected, store the fragment for later
-            if not self._upstream:
-                self._store_body(data)
-            else:
+            if self._upstream:
+                # Write the content
                 _write_to_stream(
                     self._upstream,
                     self._get_body(data),
                     is_chunked)
+            else:
+                # If we're not connected upstream, store the fragment
+                # for later
+                self._store_body(data)
 
     def on_message_complete(self, is_chunked, keep_alive):
         if self._rejected:
@@ -169,6 +172,7 @@ class UpstreamHandler(ProxyHandler):
         super(UpstreamHandler, self).__init__(filter_pl, HttpResponse())
         self._downstream = downstream
         self._upstream = upstream
+        self._chunked = False
 
     def on_status(self, status_code):
         self._http_msg.status = str(status_code)
@@ -176,10 +180,13 @@ class UpstreamHandler(ProxyHandler):
     def on_headers_complete(self):
         action = self._filter_pl.on_response_head(self._http_msg)
 
-        self._http_msg.remove_header('content-length')
-        self._http_msg.remove_header('transfer-encoding')
+        # If there's a content length, negotiate the tansfer encoding
+        if self._http_msg.get_header('content-length'):
+            self._chunked = True
+            self._http_msg.remove_header('content-length')
+            self._http_msg.remove_header('transfer-encoding')
 
-        self._http_msg.header('transfer-encoding').values.append('chunked')
+            self._http_msg.header('transfer-encoding').values.append('chunked')
 
         if action.is_rejecting():
             self._rejected = True
@@ -207,7 +214,8 @@ class UpstreamHandler(ProxyHandler):
             self._downstream.write(
                 self._http_msg.to_bytes(),
                 callback=callback)
-        else:
+
+        if is_chunked or self._chunked:
             # Finish the last chunk.
             self._downstream.write(
                 _CHUNK_CLOSE,
@@ -244,7 +252,7 @@ class ConnectionTracker(object):
         us_sock.setblocking(0)
 
         # Create and bind the IOStream
-        live_stream = tornado.iostream.IOStream(us_sock)
+        live_stream = IOStream(us_sock)
         self._streams[target] = live_stream
 
         def on_close():
@@ -272,9 +280,6 @@ class ProxyConnection(object):
             self._on_upstream_live,
             self._on_pipe_broken)
 
-        # TODO:Review - Is this kind of hold really needed? Hmmm...
-        self._hold_downstream = False
-
         # Setup all of the wiring for downstream
         self._downstream = downstream
         self._downstream_handler = DownstreamHandler(
@@ -283,8 +288,7 @@ class ProxyConnection(object):
             self._connect_upstream)
         self._downstream_parser = RequestParser(self._downstream_handler)
         self._downstream.set_close_callback(self._on_downstream_close)
-        self._downstream.read_bytes(
-            num_bytes=_MAX_READ,
+        self._downstream.read_until_close(
             callback=self._on_downstream_read,
             streaming_callback=self._on_downstream_read)
 
@@ -325,11 +329,11 @@ class ProxyConnection(object):
         # Drop the ref to the proxied request head
         self._request = None
 
-        # Allow downstream reads again
-        self._hold_downstream = False
+        # Set up our downstream handler
+        self._downstream_handler._upstream = upstream
+
         if not self._downstream.reading():
-            self._downstream.read_bytes(
-                num_bytes=_MAX_READ,
+            self._downstream.read_until_close(
                 callback=self._on_downstream_read,
                 streaming_callback=self._on_downstream_read)
 
@@ -344,29 +348,23 @@ class ProxyConnection(object):
             self._upstream_parser.destroy()
 
     def _on_downstream_read(self, data):
-        if len(data) > 0:
-            try:
-                self._downstream_parser.execute(data)
-            except iostream.StreamClosedError:
-                pass
-            except Exception as ex:
-                _LOG.exception(ex)
-        elif not self._hold_downstream:
-            self._downstream.read_bytes(
-                num_bytes=_MAX_READ,
-                callback=self._on_downstream_read,
-                streaming_callback=self._on_downstream_read)
+        try:
+            self._downstream_parser.execute(data)
+        except StreamClosedError:
+            pass
+        except Exception as ex:
+            _LOG.exception(ex)
 
     def _on_upstream_read(self, data):
         try:
             self._upstream_parser.execute(data)
-        except iostream.StreamClosedError:
+        except StreamClosedError:
             pass
         except Exception as ex:
             _LOG.exception(ex)
 
 
-class TornadoHttpProxy(tornado.tcpserver.TCPServer):
+class TornadoHttpProxy(TCPServer):
     """
     Subclass of the Tornado TCPServer that lets us set up the Pyrox proxy
     orchestrations.
